// -*- C++ -*-
#ifndef HERWIG_ShowerVariables_H
#define HERWIG_ShowerVariables_H
//
// This is the declaration of the ShowerVariables class.
//

#include "ThePEG/Interface/Interfaced.h"
#include "ShowerConfig.h"
#include "Couplings/ShowerIndex.h"
#include "ShowerVariables.fh"
#include "ThePEG/PDF/BeamParticleData.h"
#include "ThePEG/PDT/EnumParticles.h"
namespace Herwig {

using namespace ThePEG;

/** \ingroup Shower
 *
 *  This class is responsible for keeping all the constraint information 
 *  on the shower evolution. In particular, it has the scale value at 
 *  which to stop the shower. Here "scale" can be either the mass scale or the 
 *  \f$\tilde{q}\f$ (ordering variable) scale: 
 *  this class is also responsible for the
 *  conversion between these two different scale definitions.
 *
 *  Furthermore, this class can also have a veto for emission above a certain 
 *  \f$p_T\f$ scale, or a veto for emission below a certain \f$p_T\f$ scale, 
 *  where \f$p_T\f$ is the "resolution" variable. 
 *
 *  The ShowerHandler by default will decay all the unstable particles
 *  specified in the relevant interfacing during the shower.
 * 
 *  Finally, this class has also three parameters to set the low energy 
 *  cutoff mass scales for respectively QCD, QED, EWK radiation. 
 *  The class provides also set/access to the upper scale for all 
 *  interaction types and events: it is supposed to be set, at 
 *  initialization time, by some other class, to the center of mass 
 *  energy of the beam-beam interaction, and used as upper scale value 
 *  for the numerically evaluation of Sudakov form factors. 
 *
 * @see ShowerIndex
 *
 * @see \ref ShowerVariablesInterfaces "The interfaces"
 * defined for ShowerVariables.
 */
class ShowerVariables: public Interfaced {

public:

  /**
   * The default constructor.
   */
  inline ShowerVariables();

  /**
   *  Access to the various switches
   */
  //@{
  /**
   * It returns true if the particle with the specified id
   * is in the list of those that should be decayed during the showering
   * showering.
   */
  inline bool decayInShower(const long id) const;

  /**
   * It returns the low energy cutoff <em>mass </em> scale for the 
   * interaction type specified in input.
   */
  Energy cutoffMassScale(const ShowerIndex::InteractionType interaction) const;

  /**
   * It returns the low energy cutoff \f$\tilde{q}\f$ scale for the 
   * interaction type specified in input.
   */
  Energy cutoffQScale(const ShowerIndex::InteractionType interaction) const;

  /**
   * Specifies the kinematic cutoff used in the parton shower phase space. 
   */
  inline Energy kinScale() const;
  //@}

  /**
   *  Conversion between the scales.
   */
  //@{
  /**
   * Conversion between <em>mass</em> and \f$\tilde{q}\f$ scale.
   */
  inline Energy convertMassScaleToQScale(const Energy inputMassScale) const;

  /**
   * Conversion between \f$\tilde{q}\f$ and <em>mass</em> scale.
   */
  inline Energy convertQScaleToMassScale(const Energy inputQScale) const;
  //@}

  /**
   * Access/set the PDF and beam particle for the current initial-state shower
   */
  //@{
  /**
   *  Get the PDF
   */
  inline tcPDFPtr currentPDF() const;

  /**
   *  Set the PDF
   */
  inline void setCurrentPDF(tcPDFPtr);

  /**
   *  Get the beam particle data
   */
  inline Ptr<BeamParticleData>::const_pointer beamParticle() const;

  /**
   *  Set the beam particle data
   */
  inline void setBeamParticle(Ptr<BeamParticleData>::const_pointer);
  //@}

  /**
   *  Switches for matrix element corrections
   */
  //@{
  /**
   * Any ME correction?   
   */
  inline bool MECOn() const;

  /**
   * Any hard ME correction? 
   */
  inline bool hardMEC() const;

  /**
   * Any soft ME correction? 
   */
  inline bool softMEC() const;
  //@}

  /**
   * The virtuality cut-off on the gluon \f$Q_g=\frac{\delta-am_q}{b}\f$
   * @param scale The scale \f$\delta\f$
   * @param mq The quark mass \f$m_q\f$.
   */
  inline Energy kinematicCutOff(Energy scale, Energy mq) const;

  /**
   *  Set/Get the gluon mass which should be used in the reconstruction
   */
  //@{
  /**
   *  Get the mass
   */
  inline Energy gluonMass() const;

  /**
   * Set the gluon mass to the effective mass
   */
  inline void setGluonMass();
  //@}

  /**
   *  Enhancement factors for radiation needed to generate the soft matrix
   *  element correction.
   */
  //@{
  /**
   *  Access the enhancement factor for initial-state radiation
   */
  inline double initialStateRadiationEnhancementFactor() const;

  /**
   *  Access the enhancement factor for final-state radiation
   */
  inline double finalStateRadiationEnhancementFactor() const;

  /**
   *  Set the enhancement factor for initial-state radiation
   */
  inline void initialStateRadiationEnhancementFactor(double);

  /**
   *  Set the enhancement factor for final-state radiation
   */
  inline void finalStateRadiationEnhancementFactor(double);
  //@}

  /**
   * Access function for the initial conditions for the shower
   */
  //@{
  /**
   * Initial conditions for the shower of a final-final colour connection
   * - 0 is the symmetric choice
   * - 1 is maximal emmision from the coloured particle
   * - 2 is maximal emmision from the anticoloured particle
   * - 3 is randomly selected maximal emmision
   */
  inline unsigned int finalFinalConditions() const;

  /**
   * Initial conditions for the shower of an initial-final decay colour connection
   * - 0 is the symmetric choice
   * - 1 is maximal emission from the decay product
   * - 2 is the smooth choice
   */
  inline unsigned int initialFinalDecayConditions() const;

  /**
   *  Access the option denoting whether the T2 region of the decay phase
   *  space is populated by the shower (default, false) or the ME correction. 
   */
  inline bool useMEForT2() const;
  //@}

public:

  /** @name Functions used by the persistent I/O system. */
  //@{
  /**
   * Function used to write out object persistently.
   * @param os the persistent output stream written to.
   */
  void persistentOutput(PersistentOStream & os) const;

  /**
   * Function used to read in object persistently.
   * @param is the persistent input stream read from.
   * @param version the version number of the object when written.
   */
  void persistentInput(PersistentIStream & is, int version);
  //@}

  /**
   * The standard Init function used to initialize the interfaces.
   * Called exactly once for each class by the class description system
   * before the main function starts or
   * when this class is dynamically loaded.
   */
  static void Init();

public:

  /**
   * Use to initialize some scales.
   */
  static const Energy HUGEMASS; 

protected:

  /** @name Clone Methods. */
  //@{
  /**
   * Make a simple clone of this object.
   * @return a pointer to the new object.
   */
  inline virtual IBPtr clone() const;

  /** Make a clone of this object, possibly modifying the cloned object
   * to make it sane.
   * @return a pointer to the new object.
   */
  inline virtual IBPtr fullclone() const;
  //@}

protected:

  /** @name Standard Interfaced functions. */  //@{
  /**
   * Initialize this object after the setup phase before saving an
   * EventGenerator to disk.
   * @throws InitException if object could not be initialized properly.
   */
  inline virtual void doinit() throw(InitException);
  //@}

private:

  /**
   * The static object used to initialize the description of this class.
   * Indicates that this is a concrete class with persistent data.
   */
  static ClassDescription<ShowerVariables> initShowerVariables;

  /**
   * The assignment operator is private and must never be called.
   * In fact, it should not even be implemented.
   */
  ShowerVariables & operator=(const ShowerVariables &);

private:

  /** 
   * Low-energy cutoff mass scale for QCD radiation
   */
  Energy _cutoffQCDMassScale;
 
  /**
   * Low-energy cutoff mass scale for QED radiation
   */
  Energy _cutoffQEDMassScale;

  /**
   * Low-energy cutoff mass scale for EWK radiation
   */
  Energy _cutoffEWKMassScale; 

  /**
   * Kinematic cutoff used in the parton shower phase space. 
   */
  Energy _kinCutoffScale; 

  /**
   * Matrix element correction switch
   */
  int _meCorrMode; 

  /**
   *  Mass cut-off for the shower
   */
  Energy _stopShowerAtMassScale;

  /**
   *  Veto emissions above this \f$p_T\f$ scale.
   */
  Energy _vetoAbovePtScale;

  /**
   *  Veto emissions below this \f$p_T\f$ scale.
   */
  Energy _vetoBelowPtScale;

  /**
   *  PDG codes of the particles which decay during showering
   *  this is fast storage for use during running
   */
  set<long> _particlesDecayInShower;

  /**
   *  PDG codes of the particles which decay during showering
   *  this is a vector that is interfaced so they can be changed
   */
  vector<long> _inputparticlesDecayInShower;

  //@{
  /**
   *  The PDF being used for the current initial-state shower
   */
  tcPDFPtr _pdf;

  /**
   *  The beam particle data for the current initial-state shower
   */
  Ptr<BeamParticleData>::const_pointer _beam;
  //@}

  /**
   *  Parameters for the \f$Q_g=\max(\frac{\delta-am_q}{b},c)\f$ kinematic cut-off
   */
  //@{
  /**
   *  The \f$a\f$ parameter
   */
  double _a;

  /**
   *  The \f$b\f$ parameter
   */
  double _b;

  /**
   *  The \f$c\f$ parameter
   */
  Energy _c;
  //@}

  /**
   *  The gluon mass
   */
  Energy _gluonMass;

  /**
   *  Radiation enhancement factors for use with the veto algorithm
   *  if needed by the soft matrix element correction 
   */
  //@{
  /**
   *  Enhancement factor for initial-state radiation
   */
  double _initialenhance;

  /**
   *  Enhancement factor for final-state radiation
   */
  double _finalenhance;
  //@}


  /**
   *  Flags controlling the initial conditions for the shower
   */
  //@{
  /**
   * Initial conditions for the shower with a final-final colour
   * connection
   */
  unsigned int _finalFinalConditions; 

  /**
   * Initial conditions for the shower with an initial-final decay colour
   * connection. This is done according to the top decay colour 
   *  connection calculation in JHEP12(2003)_045. The options act as follows:
   *  0: This is the default 'symmetric' choice which more or less divides
   *     the phase space evenly between the parent and its charged child.
   *  1: This 'maximal' choice maximises the phase space available for 
   *     gluons emitted from the charged child.
   *  2: This (experimental) 'smooth' choice does not suffer from
   *     a discontinuity at the boundary between the region populated by
   *     emissions from the charged child and the region populated by emissions
   *     from the parent. This does, however, mean that the phase space 
   *     available for emissions from the charged child is fairly minimal.
   */
  unsigned int _initialFinalDecayConditions;

  /**
   *  This flag determines whether the T2 region in the decay shower
   *  (JHEP12(2003)_045) is populated by the ME correction (true) or
   *  the shower from the decaying particle.
   */
  bool _useMEForT2;

};

}

#include "ThePEG/Utilities/ClassTraits.h"

namespace ThePEG {

/** @cond TRAITSPECIALIZATIONS */

/** This template specialization informs ThePEG about the
 *  base classes of ShowerVariables. */
template <>
struct BaseClassTrait<Herwig::ShowerVariables,1> {
  /** Typedef of the first base class of ShowerVariables. */
  typedef Interfaced NthBase;
};

/** This template specialization informs ThePEG about the name of
 *  the ShowerVariables class and the shared object where it is defined. */
template <>
struct ClassTraits<Herwig::ShowerVariables>
  : public ClassTraitsBase<Herwig::ShowerVariables> {
  /** Return a platform-independent class name */
  static string className() { return "Herwig++::ShowerVariables"; }
  /**
   * The name of a file containing the dynamic library where the class
   * ShowerVariables is implemented. It may also include several, space-separated,
   * libraries if the class ShowerVariables depends on other classes (base classes
   * excepted). In this case the listed libraries will be dynamically
   * linked in the order they are specified.
   */
  static string library() { return "HwShower.so"; }
};

/** @endcond */

}

#include "ShowerVariables.icc"
#ifndef ThePEG_TEMPLATES_IN_CC_FILE
// #include "ShowerVariables.tcc"
#endif

#endif /* HERWIG_ShowerVariables_H */
